@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, Matias Fontanini
+ * Copyright (c) 2016, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,70 +28,99 @@
  */
 
 #include <stdexcept>
-#include <cassert>
+#include <cstring>
 #include "endianness.h"
 #include "dhcp.h"
 #include "ethernetII.h"
+#include "internals.h"
 #include "exceptions.h"
+#include "memory_helpers.h"
 
 using std::string;
+using std::vector;
 using std::list;
 using std::runtime_error;
+using std::find_if;
+
+using Tins::Memory::InputMemoryStream;
+using Tins::Memory::OutputMemoryStream;
 
 namespace Tins {
+
+PDU::metadata DHCP::extract_metadata(const uint8_t *buffer, uint32_t total_sz) {
+    if (TINS_UNLIKELY(total_sz < sizeof(bootp_header))) {
+        throw malformed_packet();
+    }
+    return metadata(total_sz, pdu_flag, PDU::UNKNOWN);
+}
+
 // Magic cookie: uint32_t. 
-DHCP::DHCP() : _size(sizeof(uint32_t)) {
+DHCP::DHCP() 
+: size_(sizeof(uint32_t)) {
     opcode(BOOTREQUEST);
     htype(1); //ethernet
     hlen(EthernetII::address_type::address_size);
 }
 
-DHCP::DHCP(const uint8_t *buffer, uint32_t total_sz) 
-: BootP(buffer, total_sz, 0), _size(sizeof(uint32_t))
-{
-    buffer += BootP::header_size() - vend().size();
-    total_sz -= BootP::header_size() - vend().size();
-    uint8_t args[2] = {0};
-    if(total_sz < sizeof(uint32_t) || *(uint32_t*)buffer != Endian::host_to_be<uint32_t>(0x63825363))
+DHCP::DHCP(const uint8_t* buffer, uint32_t total_sz) 
+: BootP(buffer, total_sz, 0), size_(sizeof(uint32_t)) {
+    InputMemoryStream stream(buffer, total_sz);
+    stream.skip(BootP::header_size() - vend().size());
+    const uint32_t magic_number = stream.read<uint32_t>();
+    if (magic_number != Endian::host_to_be<uint32_t>(0x63825363)) {
         throw malformed_packet();
-    buffer += sizeof(uint32_t);
-    total_sz -= sizeof(uint32_t);
-    while(total_sz) {
-        for(unsigned i(0); i < 2; ++i) {
-            args[i] = *(buffer++);
-            total_sz--;
-            if(args[0] == END || args[0] == PAD) {
-                args[1] = 0;
-                i = 2;
-            }
-            else if(!total_sz)
-                throw malformed_packet();
+    }
+    // While there's data left
+    while (stream) {
+        OptionTypes option_type;
+        uint8_t option_length = 0;
+        option_type = (OptionTypes)stream.read<uint8_t>();
+        // We should only read the length if it's not END nor PAD
+        if (option_type != END && option_type != PAD) {
+            option_length = stream.read<uint8_t>();
         }
-        if(total_sz < args[1])
+        // Make sure we can read the payload size
+        if (!stream.can_read(option_length)) {
             throw malformed_packet();
-        add_option(
-            option((OptionTypes)args[0], args[1], buffer)
-        );
-        buffer += args[1];
-        total_sz -= args[1];
+        }
+        add_option(option(option_type, option_length, stream.pointer()));
+        stream.skip(option_length);
     }
 }
 
-void DHCP::add_option(const option &opt) {
+void DHCP::add_option(const option& opt) {
     internal_add_option(opt);
-    _options.push_back(opt);
+    options_.push_back(opt);
 }
 
-void DHCP::internal_add_option(const option &opt) {
-    _size += opt.data_size() + (sizeof(uint8_t) << 1);
+void DHCP::internal_add_option(const option& opt) {
+    size_ += static_cast<uint32_t>(opt.data_size() + (sizeof(uint8_t) << 1));
 }
 
-const DHCP::option *DHCP::search_option(OptionTypes opt) const {
-    for(options_type::const_iterator it = _options.begin(); it != _options.end(); ++it) {
-        if(it->option() == opt)
-            return &(*it);
+bool DHCP::remove_option(OptionTypes type) {
+    options_type::iterator iter = search_option_iterator(type);
+    if (iter == options_.end()) {
+        return false;
     }
-    return 0;
+    size_ -= static_cast<uint32_t>(iter->data_size() + (sizeof(uint8_t) << 1));
+    options_.erase(iter);
+    return true;
+}
+
+const DHCP::option* DHCP::search_option(OptionTypes opt) const {
+    // Search for the iterator. If we found something, return it, otherwise return nullptr.
+    options_type::const_iterator iter = search_option_iterator(opt);
+    return (iter != options_.end()) ? &*iter : 0;
+}
+
+DHCP::options_type::const_iterator DHCP::search_option_iterator(OptionTypes opt) const {
+    Internals::option_type_equality_comparator<option> comparator(opt);
+    return find_if(options_.begin(), options_.end(), comparator);
+}
+
+DHCP::options_type::iterator DHCP::search_option_iterator(OptionTypes opt) {
+    Internals::option_type_equality_comparator<option> comparator(opt);
+    return find_if(options_.begin(), options_.end(), comparator);
 }
 
 void DHCP::type(Flags type) {
@@ -143,22 +172,22 @@ DHCP::ipaddress_type DHCP::subnet_mask() const {
     return search_and_convert<ipaddress_type>(SUBNET_MASK);
 }
 
-void DHCP::routers(const std::vector<ipaddress_type> &routers) {
+void DHCP::routers(const vector<ipaddress_type>& routers) {
     serialization_type buffer = serialize_list(routers);
     add_option(option(ROUTERS, buffer.begin(), buffer.end()));
 }
 
-std::vector<DHCP::ipaddress_type> DHCP::routers() const {
-    return search_and_convert<std::vector<DHCP::ipaddress_type> >(ROUTERS);
+vector<DHCP::ipaddress_type> DHCP::routers() const {
+    return search_and_convert<vector<DHCP::ipaddress_type> >(ROUTERS);
 }
 
-void DHCP::domain_name_servers(const std::vector<ipaddress_type> &dns) {
+void DHCP::domain_name_servers(const vector<ipaddress_type>& dns) {
     serialization_type buffer = serialize_list(dns);
     add_option(option(DOMAIN_NAME_SERVERS, buffer.begin(), buffer.end()));
 }
 
-std::vector<DHCP::ipaddress_type> DHCP::domain_name_servers() const {
-    return search_and_convert<std::vector<DHCP::ipaddress_type> >(DOMAIN_NAME_SERVERS);
+vector<DHCP::ipaddress_type> DHCP::domain_name_servers() const {
+    return search_and_convert<vector<DHCP::ipaddress_type> >(DOMAIN_NAME_SERVERS);
 }
 
 void DHCP::broadcast(ipaddress_type addr) {
@@ -179,12 +208,20 @@ DHCP::ipaddress_type DHCP::requested_ip() const {
     return search_and_convert<ipaddress_type>(DHCP_REQUESTED_ADDRESS);
 }
 
-void DHCP::domain_name(const string &name) {
+void DHCP::domain_name(const string& name) {
     add_option(option(DOMAIN_NAME, name.size(), (const uint8_t*)name.c_str()));
 }
 
-std::string DHCP::domain_name() const {
-    return search_and_convert<std::string>(DOMAIN_NAME);
+string DHCP::domain_name() const {
+    return search_and_convert<string>(DOMAIN_NAME);
+}
+
+void DHCP::hostname(const string& name) {
+    add_option(option(HOST_NAME, name.size(), (const uint8_t*)name.c_str()));
+}
+
+string DHCP::hostname() const {
+    return search_and_convert<string>(HOST_NAME);
 }
 
 void DHCP::rebind_time(uint32_t time) {
@@ -196,35 +233,35 @@ uint32_t DHCP::rebind_time() const {
     return search_and_convert<uint32_t>(DHCP_REBINDING_TIME);
 }
 
-PDU::serialization_type DHCP::serialize_list(const std::vector<ipaddress_type> &ip_list) {
+PDU::serialization_type DHCP::serialize_list(const vector<ipaddress_type>& ip_list) {
     serialization_type buffer(ip_list.size() * sizeof(uint32_t));
-    uint32_t *ptr = (uint32_t*)&buffer[0];
-    for(std::vector<ipaddress_type>::const_iterator it = ip_list.begin(); it != ip_list.end(); ++it)
+    uint32_t* ptr = (uint32_t*)&buffer[0];
+    typedef vector<ipaddress_type>::const_iterator iterator;
+    for (iterator it = ip_list.begin(); it != ip_list.end(); ++it) {
         *(ptr++) = *it;
+    }
     return buffer;
 }
 
 uint32_t DHCP::header_size() const {
-    return BootP::header_size() - vend().size() + _size;
+    return static_cast<uint32_t>(BootP::header_size() - vend().size() + size_);
 }
 
-void DHCP::write_serialization(uint8_t *buffer, uint32_t total_sz, const PDU *parent) {
-    #ifdef TINS_DEBUG
-    assert(total_sz >= header_size());
-    #endif
-    if(_size) {
-        vend_type &result(BootP::vend());
-        result.resize(_size);
-        uint8_t *ptr = &result[0] + sizeof(uint32_t);
+void DHCP::write_serialization(uint8_t* buffer, uint32_t total_sz, const PDU* parent) {
+    if (size_) {
+        vend_type& result = BootP::vend();
+        result.resize(size_);
+        // Build a stream over the vend vector
+        OutputMemoryStream stream(&result[0], result.size());
         // Magic cookie
-        *((uint32_t*)&result[0]) = Endian::host_to_be<uint32_t>(0x63825363);
-        for(options_type::const_iterator it = _options.begin(); it != _options.end(); ++it) {
-            *(ptr++) = it->option();
-            *(ptr++) = it->length_field();
-            std::copy(it->data_ptr(), it->data_ptr() + it->data_size(), ptr);
-            ptr += it->data_size();
+        stream.write(Endian::host_to_be<uint32_t>(0x63825363));
+        for (options_type::const_iterator it = options_.begin(); it != options_.end(); ++it) {
+            stream.write(it->option());
+            stream.write<uint8_t>(it->length_field());
+            stream.write(it->data_ptr(), it->data_size());
         }
     }
     BootP::write_serialization(buffer, total_sz, parent);
 }
-}
+
+} // Tins
