@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, Matias Fontanini
+ * Copyright (c) 2016, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,118 +29,132 @@
  
 #include <stdexcept>
 #include <cstring>
-#include <cassert>
 #include "dot1q.h"
 #include "internals.h"
 #include "exceptions.h"
+#include "memory_helpers.h"
+
+using Tins::Memory::InputMemoryStream;
+using Tins::Memory::OutputMemoryStream;
 
 namespace Tins {
 
+PDU::metadata Dot1Q::extract_metadata(const uint8_t *buffer, uint32_t total_sz) {
+    if (TINS_UNLIKELY(total_sz < sizeof(dot1q_header))) {
+        throw malformed_packet();
+    }
+    return metadata(sizeof(dot1q_header), pdu_flag, PDU::UNKNOWN);
+}
+
 Dot1Q::Dot1Q(small_uint<12> tag_id, bool append_pad) 
-: _header(), _append_padding(append_pad)
-{
+: header_(), append_padding_(append_pad) {
     id(tag_id);
 }
 
-Dot1Q::Dot1Q(const uint8_t *buffer, uint32_t total_sz) 
-: _append_padding()
-{
-    if(total_sz < sizeof(_header))
-        throw malformed_packet();
-    std::memcpy(&_header, buffer, sizeof(_header));
-    buffer += sizeof(_header);
-    total_sz -= sizeof(_header);
+Dot1Q::Dot1Q(const uint8_t* buffer, uint32_t total_sz) 
+: append_padding_() {
+    InputMemoryStream stream(buffer, total_sz);
+    stream.read(header_);
     
-    if(total_sz) {
+    if (stream) {
         inner_pdu(
             Internals::pdu_from_flag(
                 (Constants::Ethernet::e)payload_type(), 
-                buffer, 
-                total_sz
+                stream.pointer(), 
+                stream.size()
             )
         );
     }
 }
 
 void Dot1Q::priority(small_uint<3> new_priority) {
-    _header.priority = new_priority;
+    header_.priority = new_priority;
 }
 
 void Dot1Q::cfi(small_uint<1> new_cfi) {
-    _header.cfi = new_cfi;
+    header_.cfi = new_cfi;
 }
 
 void Dot1Q::id(small_uint<12> new_id) {
     #if TINS_IS_LITTLE_ENDIAN
-    _header.idL = new_id & 0xff;
-    _header.idH = new_id >> 8;
+    header_.idL = new_id & 0xff;
+    header_.idH = new_id >> 8;
     #else
-    _header.id = new_id;
+    header_.id = new_id;
     #endif
 }
 
 void Dot1Q::payload_type(uint16_t new_type) {
-    _header.type = Endian::host_to_be(new_type);
+    header_.type = Endian::host_to_be(new_type);
 }
 
 uint32_t Dot1Q::header_size() const {
-    return sizeof(_header);
+    return sizeof(header_);
 }
 
 uint32_t Dot1Q::trailer_size() const {
-    if(_append_padding) {
-        uint32_t total_size = sizeof(_header);
-        if(inner_pdu())
+    if (append_padding_) {
+        uint32_t total_size = sizeof(header_);
+        if (inner_pdu()) {
             total_size += inner_pdu()->size();
+        }
         return (total_size > 50) ? 0 : (50 - total_size);
     }
-    else
+    else {
         return 0;
+    }
 }
 
-void Dot1Q::write_serialization(uint8_t *buffer, uint32_t total_sz, const PDU *) {
-    uint32_t trailer = trailer_size();
-    #ifdef TINS_DEBUG
-    assert(total_sz >= sizeof(_header) + trailer);
-    #endif
-    if ((payload_type() == 0) && inner_pdu()) {
-        Constants::Ethernet::e flag = Internals::pdu_flag_to_ether_type(
-            inner_pdu()->pdu_type()
-        );
+void Dot1Q::write_serialization(uint8_t* buffer, uint32_t total_sz, const PDU *) {
+    OutputMemoryStream stream(buffer, total_sz);
+    if (inner_pdu()) {
+        Constants::Ethernet::e flag;
+        PDUType type = inner_pdu()->pdu_type();
+        if (type == PDU::DOT1Q) {
+            flag = Constants::Ethernet::QINQ;
+        }
+        else {
+            // Set the appropriate payload type flag
+            flag = Internals::pdu_flag_to_ether_type(type);
+        }
         payload_type(static_cast<uint16_t>(flag));
     }
-    std::memcpy(buffer, &_header, sizeof(_header));
-    
-    buffer += sizeof(_header);
-    if(inner_pdu())
-        buffer += inner_pdu()->size();
-    std::fill(buffer, buffer + trailer, 0);
+    stream.write(header_);
+
+    // Skip inner PDU size
+    if (inner_pdu()) {
+        stream.skip(inner_pdu()->size());
+    }
+    // Write trailer
+    stream.fill(trailer_size(), 0);
 }
 
 #if TINS_IS_LITTLE_ENDIAN
-    uint16_t Dot1Q::get_id(const dot1q_hdr *hdr) {
+    uint16_t Dot1Q::get_id(const dot1q_header* hdr) {
         return hdr->idL | (hdr->idH << 8);
     }
 #else
-    uint16_t Dot1Q::get_id(const dot1q_hdr *hdr) {
+    uint16_t Dot1Q::get_id(const dot1q_header* hdr) {
         return hdr->id;
     }
 #endif
 
 void Dot1Q::append_padding(bool value) {
-    _append_padding = value;
+    append_padding_ = value;
 }
 
-bool Dot1Q::matches_response(const uint8_t *ptr, uint32_t total_sz) const {
-    if(total_sz < sizeof(_header))
+bool Dot1Q::matches_response(const uint8_t* ptr, uint32_t total_sz) const {
+    if (total_sz < sizeof(header_)) {
         return false;
-    const dot1q_hdr *dot1q_ptr = (const dot1q_hdr*)ptr;
-    if(get_id(dot1q_ptr) == get_id(&_header)) {
-        ptr += sizeof(_header);
-        total_sz -= sizeof(_header);
+    }
+    const dot1q_header* dot1q_ptr = (const dot1q_header*)ptr;
+    if (get_id(dot1q_ptr) == get_id(&header_)) {
+        ptr += sizeof(header_);
+        total_sz -= sizeof(header_);
         return inner_pdu() ? inner_pdu()->matches_response(ptr, total_sz) : true;
     }
     return false;
     
 }
-}
+
+} // Tins
