@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, Matias Fontanini
+ * Copyright (c) 2016, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,6 +27,12 @@
  *
  */
 
+#ifdef _WIN32
+    #include <ws2tcpip.h>
+#else // _WIN32
+    #include <sys/socket.h>
+    #include <arpa/inet.h>
+#endif // _WIN32
 #include <stdexcept>
 #include <sstream>
 #include "ip_address.h"
@@ -34,6 +40,8 @@
 #include "address_range.h"
 
 using std::string;
+using std::ostringstream;
+using std::ostream;
 
 namespace Tins{
 const IPv4Address IPv4Address::broadcast("255.255.255.255");
@@ -48,70 +56,68 @@ const AddressRange<IPv4Address> loopback_range = IPv4Address("127.0.0.0") / 8;
 const AddressRange<IPv4Address> multicast_range = IPv4Address("224.0.0.0") / 4;
     
 IPv4Address::IPv4Address(uint32_t ip) 
-: ip_addr(Endian::be_to_host(ip)) {
+: ip_addr_(Endian::be_to_host(ip)) {
     
 }
 
-IPv4Address::IPv4Address(const char *ip) {
-    ip_addr = ip ? ip_to_int(ip) : 0; 
+IPv4Address::IPv4Address(const char* ip) {
+    ip_addr_ = ip ? ip_to_int(ip) : 0; 
 }
 
-IPv4Address::IPv4Address(const std::string &ip) 
-: ip_addr(ip_to_int(ip)) {
+IPv4Address::IPv4Address(const std::string& ip) 
+: ip_addr_(ip_to_int(ip.c_str())) {
       
 } 
 
 IPv4Address::operator uint32_t() const { 
-    return Endian::host_to_be(ip_addr); 
+    return Endian::host_to_be(ip_addr_); 
 }
 
-std::string IPv4Address::to_string() const {
-    std::ostringstream oss;
-    oss << *this;
+string IPv4Address::to_string() const {
+    ostringstream oss;
+    oss <<* this;
     return oss.str();
 }
 
-uint32_t IPv4Address::ip_to_int(const string &ip) {
-    uint32_t result(0), i(0), end, bytes_found(0);
-    while(i < ip.size() && bytes_found < 4) {
-        uint16_t this_byte(0);
-        end = i + 3;
-        while(i < ip.size() && i < end && ip[i] != '.') {
-            if(ip[i] < '0' || ip[i] > '9')
-                throw std::runtime_error("Non-digit character found in ip");
-            this_byte = (this_byte * 10)  + (ip[i] - '0');
-            i++;
+uint32_t IPv4Address::ip_to_int(const char* ip) {
+    #ifdef _WIN32
+        in_addr addr;
+        if (InetPtonA(AF_INET, ip, &addr)) {
+            return Endian::be_to_host(addr.s_addr);
         }
-        if (this_byte > 0xFF) {
-            throw std::runtime_error("Byte greater than 255");
+        else {
+            throw invalid_address();
         }
-        result = (result << 8) | (this_byte & 0xFF);
-        bytes_found++;
-        if(bytes_found < 4 && i < ip.size() && ip[i] == '.')
-            i++;
-    }
-    if(bytes_found < 4 || (i < ip.size() && bytes_found == 4))
-        throw std::runtime_error("Invalid ip address");
-    return result;
+    #else // _WIN32
+        in_addr addr;
+        if (inet_pton(AF_INET, ip, &addr) == 1) {
+            return Endian::be_to_host(addr.s_addr);
+        }
+        else {
+            throw invalid_address();
+        }
+    #endif // _WIN32
 }
 
-std::ostream &operator<<(std::ostream &output, const IPv4Address &addr) {
+ostream& operator<<(ostream& output, const IPv4Address& addr) {
     int mask(24);
-    uint32_t ip_addr = addr.ip_addr;
-    while(mask >=0) {
-        output << ((ip_addr >> mask) & 0xff);
-        if(mask)
+    uint32_t ip_addr_ = addr.ip_addr_;
+    while (mask >=0) {
+        output << ((ip_addr_ >> mask) & 0xff);
+        if (mask) {
             output <<  '.';
+        }
         mask -= 8;
     }
-    return output;;
+    return output;
 }
 
 bool IPv4Address::is_private() const {
-    const AddressRange<IPv4Address> *iter = private_ranges;
-    while(iter != private_ranges + 3) {
-        if(iter->contains(*this))
+    const AddressRange<IPv4Address>* iter = private_ranges;
+    while (iter != private_ranges + 3) {
+        if (iter->contains(*this)) {
             return true;
+        }
         ++iter;
     }
     return false;
@@ -130,6 +136,7 @@ bool IPv4Address::is_unicast() const {
 }
 
 bool IPv4Address::is_broadcast() const {
-    return *this == broadcast;
-}
+    return* this == broadcast;
 }
+
+} // Tins
