@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, Matias Fontanini
+ * Copyright (c) 2016, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,18 +27,17 @@
  *
  */
 
-#ifndef WIN32
+#ifndef _WIN32
     #include <sys/socket.h>
     #ifdef BSD
         #include <net/if_dl.h>
         #include <netinet/in.h>
         #include <net/ethernet.h>
     #endif
+#else
+    #include <ws2tcpip.h>
 #endif
 #include <stdexcept>
-#ifdef TINS_DEBUG
-#include <cassert>
-#endif
 #include <cstring>
 #include "loopback.h"
 #include "packet_sender.h"
@@ -46,68 +45,84 @@
 #include "llc.h"
 #include "rawpdu.h"
 #include "exceptions.h"
+#include "memory_helpers.h"
 
 #if !defined(PF_LLC)
     // compilation fix, nasty but at least works on BSD
     #define PF_LLC 26
 #endif
 
+using Tins::Memory::InputMemoryStream;
+using Tins::Memory::OutputMemoryStream;
+
 namespace Tins {
+
 Loopback::Loopback()
-: _family()
-{
+: family_() {
     
 }
 
-Loopback::Loopback(const uint8_t *buffer, uint32_t total_sz) 
-{
-    if(total_sz < sizeof(_family))
-        throw malformed_packet();
-    _family = *reinterpret_cast<const uint32_t*>(buffer);
-    buffer += sizeof(uint32_t);
-    total_sz -= sizeof(uint32_t);
-    #ifndef WIN32
-    if(total_sz) {
-        switch(_family) {
+Loopback::Loopback(const uint8_t* buffer, uint32_t total_sz) {
+    InputMemoryStream stream(buffer, total_sz);
+    family_ = stream.read<uint32_t>();
+    #ifndef _WIN32
+    if (total_sz) {
+        switch (family_) {
             case PF_INET:
-                inner_pdu(new Tins::IP(buffer, total_sz));
+                inner_pdu(new Tins::IP(stream.pointer(), stream.size()));
                 break;
             case PF_LLC:
-                inner_pdu(new Tins::LLC(buffer, total_sz));
+                inner_pdu(new Tins::LLC(stream.pointer(), stream.size()));
                 break;
             default:
-                inner_pdu(new Tins::RawPDU(buffer, total_sz));
+                inner_pdu(new Tins::RawPDU(stream.pointer(), stream.size()));
                 break;
         };
     }
-    #endif // WIN32
+    #endif // _WIN32
 }
     
 void Loopback::family(uint32_t family_id) {
-    _family = family_id;
+    family_ = family_id;
 }
 
 uint32_t Loopback::header_size() const {
-    return sizeof(_family);
+    return sizeof(family_);
 }
 
-void Loopback::write_serialization(uint8_t *buffer, uint32_t total_sz, const PDU *) 
-{
-    #ifdef TINS_DEBUG
-    assert(total_sz >= sizeof(_family));
-    #endif
-    if(tins_cast<const Tins::IP*>(inner_pdu()))
-        _family = PF_INET;
-    else if(tins_cast<const Tins::LLC*>(inner_pdu()))
-        _family = PF_LLC;
-    *reinterpret_cast<uint32_t*>(buffer) = _family;
+void Loopback::write_serialization(uint8_t* buffer, uint32_t total_sz, const PDU *) {
+    OutputMemoryStream stream(buffer, total_sz);
+    #ifndef _WIN32
+    if (tins_cast<const Tins::IP*>(inner_pdu())) {
+        family_ = PF_INET;
+    }
+    else if (tins_cast<const Tins::LLC*>(inner_pdu())) {
+        family_ = PF_LLC;
+    }
+    stream.write(family_);
+    #endif // _WIN32
 }
+
+bool Loopback::matches_response(const uint8_t* ptr, uint32_t total_sz) const {
+    if (total_sz < sizeof(family_)) {
+        return false;
+    }
+    // If there's an inner_pdu, check if the inner pdu matches.
+    // Otherwise, just check this loopback family.
+    
+    return inner_pdu() ? 
+           inner_pdu()->matches_response(ptr + sizeof(family_), total_sz - sizeof(family_)) :
+           (family_ == *reinterpret_cast<const uint32_t*>(ptr));
+}
+
 #ifdef BSD
-void Loopback::send(PacketSender &sender, const NetworkInterface &iface) {
-    if(!iface)
+void Loopback::send(PacketSender& sender, const NetworkInterface& iface) {
+    if (!iface) {
         throw invalid_interface();
+    }
     
     sender.send_l2(*this, 0, 0, iface);
 }
-#endif // WIN32
-}
+#endif // _WIN32
+
+} // Tins
