@@ -20,8 +20,8 @@ public:
     static const uint8_t sname[], file[];
     static const IPv4Address addr;
 
-    void test_equals(const DHCP &dhcp1, const DHCP &dhcp2);
-    void test_option(const DHCP &dhcp, DHCP::OptionTypes opt, uint32_t len = 0, uint8_t *value = 0);
+    void test_equals(const DHCP& dhcp1, const DHCP& dhcp2);
+    void test_option(const DHCP& dhcp, DHCP::OptionTypes opt, uint32_t len = 0, uint8_t* value = 0);
 };
 
 const BootP::chaddr_type DHCPTest::chaddr("16:ab:54:12:fa:ca:56:7f:1b:65:11:fa:da:ab:19:18");
@@ -167,8 +167,8 @@ TEST_F(DHCPTest, File) {
     EXPECT_TRUE(memcmp(dhcp.file(), file, 128) == 0);
 }
 
-void DHCPTest::test_option(const DHCP &dhcp, DHCP::OptionTypes opt, uint32_t len, uint8_t *value) {
-    const DHCP::option *option = dhcp.search_option(opt);
+void DHCPTest::test_option(const DHCP& dhcp, DHCP::OptionTypes opt, uint32_t len, uint8_t* value) {
+    const DHCP::option* option = dhcp.search_option(opt);
     ASSERT_TRUE(option != 0);
     EXPECT_EQ(option->option(), opt);
     ASSERT_EQ(option->data_size(), len);
@@ -225,11 +225,18 @@ TEST_F(DHCPTest, DNSOption) {
 
 TEST_F(DHCPTest, DomainNameOption) {
     DHCP dhcp;
-    string domain = "libtins.test.domain", domain_found;
+    string domain = "libtins.test.domain";
     dhcp.domain_name(domain);
     EXPECT_EQ(domain, dhcp.domain_name());
 }
 
+TEST_F(DHCPTest, HostnameOption) {
+    DHCP dhcp;
+    string hostname = "libtins-hostname";
+    dhcp.hostname(hostname);
+    EXPECT_EQ(hostname, dhcp.hostname());
+}
+
 TEST_F(DHCPTest, BroadcastOption) {
     DHCP dhcp;
     IPv4Address ip = "192.168.0.1", ip_found;
@@ -237,7 +244,7 @@ TEST_F(DHCPTest, BroadcastOption) {
     EXPECT_EQ(ip, dhcp.broadcast());
 }
 
-void DHCPTest::test_equals(const DHCP &dhcp1, const DHCP &dhcp2) {
+void DHCPTest::test_equals(const DHCP& dhcp1, const DHCP& dhcp2) {
     EXPECT_EQ(dhcp1.opcode(), dhcp2.opcode());
     EXPECT_EQ(dhcp1.htype(), dhcp2.htype());
     ASSERT_EQ(dhcp1.hlen(), dhcp2.hlen());
@@ -269,7 +276,7 @@ TEST_F(DHCPTest, ConstructorFromBuffer) {
     DHCP dhcp1(expected_packet, sizeof(expected_packet));
     std::vector<IPv4Address> routers, expected_routers;
     expected_routers.push_back("192.168.0.1");
-     expected_routers.push_back("127.0.0.1");
+    expected_routers.push_back("127.0.0.1");
 
     EXPECT_EQ(dhcp1.opcode(), DHCP::DISCOVER);
     EXPECT_EQ(dhcp1.htype(), 1);
@@ -294,8 +301,19 @@ TEST_F(DHCPTest, Serialize) {
     ASSERT_EQ(buffer.size(), sizeof(expected_packet));
     EXPECT_TRUE(std::equal(buffer.begin(), buffer.end(), expected_packet));
 
-    DHCP dhcp2(&buffer[0], buffer.size());
+    DHCP dhcp2(&buffer[0], (uint32_t)buffer.size());
     test_equals(dhcp1, dhcp2);
 }
 
+TEST_F(DHCPTest, RemoveOption) {
+    DHCP dhcp;
+    PDU::serialization_type old_buffer = dhcp.serialize();
+    dhcp.domain_name("libtins.github.io");
+    dhcp.server_identifier("192.168.0.1");
 
+    EXPECT_TRUE(dhcp.remove_option(DHCP::DOMAIN_NAME));
+    EXPECT_TRUE(dhcp.remove_option(DHCP::DHCP_SERVER_IDENTIFIER));
+
+    PDU::serialization_type new_buffer = dhcp.serialize();
+    EXPECT_EQ(old_buffer, new_buffer);
+}
