@@ -1,5 +1,5 @@
 #include "dot11/dot11_control.h"
-#ifdef HAVE_DOT11
+#ifdef TINS_HAVE_DOT11
 
 #include <gtest/gtest.h>
 #include "tests/dot11.h"
@@ -22,14 +22,14 @@ const uint8_t Dot11AckTest::expected_packet[] = {
     213, 1, 79, 35, 0, 1, 2, 3, 4, 5
 };
 
-void test_equals(const Dot11Ack &dot1, const Dot11Ack &dot2) {
+void test_equals(const Dot11Ack& dot1, const Dot11Ack& dot2) {
     test_equals(
         static_cast<const Dot11&>(dot1),
         static_cast<const Dot11&>(dot2)
     );
 }
 
-void test_equals_expected(const Dot11Ack &dot11) {
+void test_equals_expected(const Dot11Ack& dot11) {
     EXPECT_EQ(dot11.protocol(), 1);
     EXPECT_EQ(dot11.type(), Dot11::CONTROL);
     EXPECT_EQ(dot11.subtype(), Dot11::ACK);
@@ -88,9 +88,9 @@ TEST_F(Dot11AckTest, ClonePDU) {
 
 TEST_F(Dot11AckTest, FromBytes) {
     Internals::smart_ptr<PDU>::type dot11(Dot11::from_bytes(expected_packet, sizeof(expected_packet)));
-    ASSERT_TRUE(dot11.get());
-    const Dot11Ack *inner = dot11->find_pdu<Dot11Ack>();
-    ASSERT_TRUE(inner);
+    ASSERT_TRUE(dot11.get() != NULL);
+    const Dot11Ack* inner = dot11->find_pdu<Dot11Ack>();
+    ASSERT_TRUE(inner != NULL);
     test_equals_expected(*inner);
 }
 
@@ -101,4 +101,4 @@ TEST_F(Dot11AckTest, Serialize) {
     EXPECT_TRUE(std::equal(buffer.begin(), buffer.end(), expected_packet));
 }
 
-#endif // HAVE_DOT11
+#endif // TINS_HAVE_DOT11
