@@ -1,6 +1,6 @@
 #include "dot11/dot11_assoc.h"
 
-#ifdef HAVE_DOT11
+#ifdef TINS_HAVE_DOT11
 
 #include <gtest/gtest.h>
 #include "tests/dot11_mgmt.h"
@@ -22,7 +22,7 @@ const uint8_t Dot11AssocResponseTest::expected_packet[] = {
     0, 0, 21, 32, 243, 146, 58, 242
 };
 
-void test_equals(const Dot11AssocResponse &dot1, const Dot11AssocResponse &dot2) {
+void test_equals(const Dot11AssocResponse& dot1, const Dot11AssocResponse& dot2) {
     test_equals(dot1.capabilities(), dot2.capabilities());
     EXPECT_EQ(dot1.status_code(), dot2.status_code());
     EXPECT_EQ(dot1.aid(), dot2.aid());
@@ -32,7 +32,7 @@ void test_equals(const Dot11AssocResponse &dot1, const Dot11AssocResponse &dot2)
     );
 }
 
-void test_equals_expected(const Dot11AssocResponse &dot11) {
+void test_equals_expected(const Dot11AssocResponse& dot11) {
     test_equals_expected(static_cast<const Dot11ManagementFrame&>(dot11));
     EXPECT_EQ(dot11.status_code(), 0x92f3);
     EXPECT_EQ(dot11.aid(), 0xf23a);
@@ -85,9 +85,9 @@ TEST_F(Dot11AssocResponseTest, ClonePDU) {
 
 TEST_F(Dot11AssocResponseTest, FromBytes) {
     Internals::smart_ptr<PDU>::type dot11(Dot11::from_bytes(expected_packet, sizeof(expected_packet)));
-    ASSERT_TRUE(dot11.get());
-    const Dot11AssocResponse *inner = dot11->find_pdu<Dot11AssocResponse>();
-    ASSERT_TRUE(inner);
+    ASSERT_TRUE(dot11.get() != NULL);
+    const Dot11AssocResponse* inner = dot11->find_pdu<Dot11AssocResponse>();
+    ASSERT_TRUE(inner != NULL);
     test_equals_expected(*inner);
 }
 
@@ -98,4 +98,4 @@ TEST_F(Dot11AssocResponseTest, Serialize) {
     EXPECT_TRUE(std::equal(buffer.begin(), buffer.end(), expected_packet));
 }
 
-#endif // HAVE_DOT11
+#endif // TINS_HAVE_DOT11
