@@ -1,6 +1,6 @@
 #include "config.h"
 
-#ifdef HAVE_DOT11
+#ifdef TINS_HAVE_DOT11
 
 #include <gtest/gtest.h>
 #include "tests/dot11_data.h"
@@ -103,9 +103,9 @@ TEST_F(Dot11DataTest, ClonePDU) {
 
 TEST_F(Dot11DataTest, FromBytes) {
     Internals::smart_ptr<PDU>::type dot11(Dot11::from_bytes(expected_packet, sizeof(expected_packet)));
-    ASSERT_TRUE(dot11.get());
-    const Dot11Data *inner = dot11->find_pdu<Dot11Data>();
-    ASSERT_TRUE(inner);
+    ASSERT_TRUE(dot11.get() != NULL);
+    const Dot11Data* inner = dot11->find_pdu<Dot11Data>();
+    ASSERT_TRUE(inner != NULL);
     test_equals_expected(*inner);
 }
 
@@ -168,4 +168,4 @@ TEST_F(Dot11DataTest, Source_Dest_BSSID_Address3) {
     EXPECT_EQ(data.bssid_addr(), "00:18:f8:f5:c2:c6");
 }
 
-#endif // HAVE_DOT11
+#endif // TINS_HAVE_DOT11
