@@ -1,6 +1,6 @@
 #include "dot11/dot11_auth.h"
 
-#ifdef HAVE_DOT11
+#ifdef TINS_HAVE_DOT11
 
 #include <gtest/gtest.h>
 #include "tests/dot11_mgmt.h"
@@ -22,7 +22,7 @@ const uint8_t Dot11DeauthenticationTest::expected_packet[] = {
     7, 0, 0, 243, 146
 };
 
-void test_equals(const Dot11Deauthentication &dot1, const Dot11Deauthentication &dot2) {
+void test_equals(const Dot11Deauthentication& dot1, const Dot11Deauthentication& dot2) {
     EXPECT_EQ(dot1.reason_code(), dot2.reason_code());
     test_equals(
         static_cast<const Dot11ManagementFrame&>(dot1),
@@ -30,7 +30,7 @@ void test_equals(const Dot11Deauthentication &dot1, const Dot11Deauthentication
     );
 }
 
-void test_equals_expected(const Dot11Deauthentication &dot11) {
+void test_equals_expected(const Dot11Deauthentication& dot11) {
     test_equals_expected(static_cast<const Dot11ManagementFrame&>(dot11));
     EXPECT_EQ(dot11.reason_code(), 0x92f3);
     EXPECT_EQ(dot11.subtype(), Dot11::DEAUTH);
@@ -75,9 +75,9 @@ TEST_F(Dot11DeauthenticationTest, ClonePDU) {
 
 TEST_F(Dot11DeauthenticationTest, FromBytes) {
     Internals::smart_ptr<PDU>::type dot11(Dot11::from_bytes(expected_packet, sizeof(expected_packet)));
-    ASSERT_TRUE(dot11.get());
-    const Dot11Deauthentication *inner = dot11->find_pdu<Dot11Deauthentication>();
-    ASSERT_TRUE(inner);
+    ASSERT_TRUE(dot11.get() != NULL);
+    const Dot11Deauthentication* inner = dot11->find_pdu<Dot11Deauthentication>();
+    ASSERT_TRUE(inner != NULL);
     test_equals_expected(*inner);
 }
 
@@ -88,4 +88,4 @@ TEST_F(Dot11DeauthenticationTest, Serialize) {
     EXPECT_TRUE(std::equal(buffer.begin(), buffer.end(), expected_packet));
 }
 
-#endif // HAVE_DOT11
+#endif // TINS_HAVE_DOT11
