@@ -1,6 +1,6 @@
 #include "dot11/dot11_probe.h"
 
-#ifdef HAVE_DOT11
+#ifdef TINS_HAVE_DOT11
 
 #include <gtest/gtest.h>
 #include "tests/dot11_mgmt.h"
@@ -22,7 +22,7 @@ const uint8_t Dot11ProbeResponseTest::expected_packet[] = {
     0, 0, 145, 138, 131, 39, 223, 152, 166, 23, 141, 146, 0, 0
 };
 
-void test_equals(const Dot11ProbeResponse &dot1, const Dot11ProbeResponse &dot2) {
+void test_equals(const Dot11ProbeResponse& dot1, const Dot11ProbeResponse& dot2) {
     EXPECT_EQ(dot1.interval(), dot2.interval());
     EXPECT_EQ(dot1.timestamp(), dot2.timestamp());
     test_equals(
@@ -31,7 +31,7 @@ void test_equals(const Dot11ProbeResponse &dot1, const Dot11ProbeResponse &dot2)
     );
 }
 
-void test_equals_expected(const Dot11ProbeResponse &dot11) {
+void test_equals_expected(const Dot11ProbeResponse& dot11) {
     test_equals_expected(static_cast<const Dot11ManagementFrame&>(dot11));
     EXPECT_EQ(dot11.timestamp(), 0x17a698df27838a91ULL);
     EXPECT_EQ(dot11.interval(), 0x928d);
@@ -84,9 +84,9 @@ TEST_F(Dot11ProbeResponseTest, ClonePDU) {
 
 TEST_F(Dot11ProbeResponseTest, FromBytes) {
     Internals::smart_ptr<PDU>::type dot11(Dot11::from_bytes(expected_packet, sizeof(expected_packet)));
-    ASSERT_TRUE(dot11.get());
-    const Dot11ProbeResponse *inner = dot11->find_pdu<Dot11ProbeResponse>();
-    ASSERT_TRUE(inner);
+    ASSERT_TRUE(dot11.get() != NULL);
+    const Dot11ProbeResponse* inner = dot11->find_pdu<Dot11ProbeResponse>();
+    ASSERT_TRUE(inner != NULL);
     test_equals_expected(*inner);
 }
 
@@ -97,4 +97,4 @@ TEST_F(Dot11ProbeResponseTest, Serialize) {
     EXPECT_TRUE(std::equal(buffer.begin(), buffer.end(), expected_packet));
 }
 
-#endif // HAVE_DOT11
+#endif // TINS_HAVE_DOT11
