@@ -1,6 +1,6 @@
 #include "config.h"
 
-#ifdef HAVE_DOT11
+#ifdef TINS_HAVE_DOT11
 
 #include <gtest/gtest.h>
 #include "tests/dot11_control.h"
@@ -21,14 +21,14 @@ const uint8_t Dot11RTSTest::expected_packet[] = {
     181, 1, 79, 35, 0, 1, 2, 3, 4, 5, 1, 2, 3, 4, 5, 6
 };
 
-void test_equals(const Dot11RTS &dot1, const Dot11RTS &dot2) {
+void test_equals(const Dot11RTS& dot1, const Dot11RTS& dot2) {
     test_equals(
         static_cast<const Dot11ControlTA&>(dot1),
         static_cast<const Dot11ControlTA&>(dot2)
     );
 }
 
-void test_equals_expected(const Dot11RTS &dot11) {
+void test_equals_expected(const Dot11RTS& dot11) {
     test_equals_expected(static_cast<const Dot11ControlTA&>(dot11));
     EXPECT_EQ(dot11.subtype(), Dot11::RTS);
 }
@@ -65,9 +65,9 @@ TEST_F(Dot11RTSTest, ClonePDU) {
 
 TEST_F(Dot11RTSTest, FromBytes) {
     Internals::smart_ptr<PDU>::type dot11(Dot11::from_bytes(expected_packet, sizeof(expected_packet)));
-    ASSERT_TRUE(dot11.get());
-    const Dot11RTS *inner = dot11->find_pdu<Dot11RTS>();
-    ASSERT_TRUE(inner);
+    ASSERT_TRUE(dot11.get() != NULL);
+    const Dot11RTS* inner = dot11->find_pdu<Dot11RTS>();
+    ASSERT_TRUE(inner != NULL);
     test_equals_expected(*inner);
 }
 
@@ -78,4 +78,4 @@ TEST_F(Dot11RTSTest, Serialize) {
     EXPECT_TRUE(std::equal(buffer.begin(), buffer.end(), expected_packet));
 }
 
-#endif // HAVE_DOT11
+#endif // TINS_HAVE_DOT11
