@@ -20,7 +20,7 @@ public:
     static address_type empty_addr;
     static const uint16_t p_type;
 
-    void test_equals(const EthernetII &eth1, const EthernetII &eth2);
+    void test_equals(const EthernetII& eth1, const EthernetII& eth2);
 };
 
 const uint8_t EthernetIITest::expected_packet[] = {
@@ -53,11 +53,11 @@ address_type EthernetIITest::empty_addr;
 
 const uint16_t EthernetIITest::p_type = 0xd0ab;
 
-void EthernetIITest::test_equals(const EthernetII &eth1, const EthernetII &eth2) {
+void EthernetIITest::test_equals(const EthernetII& eth1, const EthernetII& eth2) {
     EXPECT_EQ(eth1.dst_addr(), eth2.dst_addr());
     EXPECT_EQ(eth1.src_addr(), eth2.src_addr());
     EXPECT_EQ(eth1.payload_type(), eth2.payload_type());
-    EXPECT_EQ((bool)eth1.inner_pdu(), (bool)eth2.inner_pdu());
+    EXPECT_EQ(eth1.inner_pdu() != NULL, eth2.inner_pdu() != NULL);
 }
 
 TEST_F(EthernetIITest, DefaultConstructor) {
@@ -83,7 +83,7 @@ TEST_F(EthernetIITest, CopyAssignmentOperator) {
 }
 
 TEST_F(EthernetIITest, NestedCopy) {
-    EthernetII *nested = new EthernetII(expected_packet, sizeof(expected_packet));
+    EthernetII* nested = new EthernetII(expected_packet, sizeof(expected_packet));
     EthernetII eth1(expected_packet, sizeof(expected_packet));
     eth1.inner_pdu(nested);
     EthernetII eth2(eth1);
@@ -126,7 +126,7 @@ TEST_F(EthernetIITest, Serialize) {
 
 TEST_F(EthernetIITest, SerializeSmallEthernetWithPadding) {
     EthernetII eth(smallip_packet, sizeof(smallip_packet));
-    ASSERT_TRUE(eth.inner_pdu());
+    ASSERT_TRUE(eth.inner_pdu() != NULL);
     PDU::serialization_type serialized = eth.serialize();
     EXPECT_EQ(serialized.size(), sizeof(smallip_packet));
     EXPECT_TRUE(std::equal(serialized.begin(), serialized.end(), smallip_packet));
@@ -141,20 +141,30 @@ TEST_F(EthernetIITest, ConstructorFromBuffer) {
 
 TEST_F(EthernetIITest, ConstructorFromIPBuffer) {
     EthernetII eth(ip_packet, sizeof(ip_packet));
-    ASSERT_TRUE(eth.inner_pdu());
+    ASSERT_TRUE(eth.inner_pdu() != NULL);
     EXPECT_EQ(eth.find_pdu<IP>(), eth.inner_pdu());
 }
 
 TEST_F(EthernetIITest, ConstructorFromIPv6Buffer) {
     EthernetII eth(ipv6_packet, sizeof(ipv6_packet));
-    ASSERT_TRUE(eth.inner_pdu());
+    ASSERT_TRUE(eth.inner_pdu() != NULL);
     EXPECT_EQ(eth.find_pdu<IPv6>(), eth.inner_pdu());
 }
 
 TEST_F(EthernetIITest, EliminateEthernetPadding) {
     EthernetII eth(smallip_packet, sizeof(smallip_packet));
-    ASSERT_TRUE(eth.inner_pdu());
-    ASSERT_TRUE(eth.find_pdu<IP>());
-    ASSERT_TRUE(eth.find_pdu<TCP>());
-    ASSERT_FALSE(eth.find_pdu<RawPDU>());
+    ASSERT_TRUE(eth.inner_pdu() != NULL);
+    ASSERT_TRUE(eth.find_pdu<IP>() != NULL);
+    ASSERT_TRUE(eth.find_pdu<TCP>() != NULL);
+    ASSERT_FALSE(eth.find_pdu<RawPDU>() != NULL);
 }
+
+TEST_F(EthernetIITest, SerializePreservesGivenPayloadType) {
+    EthernetII eth;
+    uint32_t payload_size = 100;
+    uint8_t payload[100] = {0};
+    eth /= RawPDU(payload, payload_size);
+    eth.payload_type(p_type);
+    eth.serialize();
+    EXPECT_EQ(p_type, eth.payload_type());
+}
\ No newline at end of file
