@@ -4,6 +4,7 @@
 #include <stdint.h>
 #include "pppoe.h"
 #include "ethernetII.h"
+#include "rawpdu.h"
 
 using namespace std;
 using namespace Tins;
@@ -11,12 +12,35 @@ using namespace Tins;
 class PPPoETest : public testing::Test {
 public:
     static const uint8_t expected_packet[];
+    static const uint8_t session_packet[];
+    static const uint8_t full_session_packet[];
+    static const uint8_t full_session_packet2[];
 };
 
 const uint8_t PPPoETest::expected_packet[] = {
     17, 9, 0, 0, 0, 16, 1, 1, 0, 0, 1, 2, 0, 0, 1, 3, 0, 4, 97, 98, 99, 100
 };
 
+const uint8_t PPPoETest::session_packet[] = {
+    17, 0, 0, 98, 0, 21, 192, 33, 1, 11, 0, 19, 1, 4, 5, 212, 3, 5, 
+    194, 35, 5, 5, 6, 22, 173, 224, 50, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0
+};
+
+const uint8_t PPPoETest::full_session_packet[] = {
+    0, 5, 133, 192, 164, 17, 0, 144, 26, 65, 118, 126, 136, 100, 17, 
+    0, 0, 98, 0, 21, 192, 33, 1, 11, 0, 19, 1, 4, 5, 212, 3, 5, 194, 
+    35, 5, 5, 6, 22, 173, 224, 50, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+    , 0, 0, 0, 0, 0, 0, 0
+};
+
+const uint8_t PPPoETest::full_session_packet2[] = {
+    255, 255, 255, 255, 255, 255, 0, 12, 41, 87, 232, 60, 136, 100, 17, 
+    0, 0, 0, 0, 50, 0, 87, 96, 0, 0, 0, 0, 8, 58, 1, 254, 128, 0, 0, 0, 
+    0, 0, 0, 2, 12, 41, 255, 254, 87, 232, 60, 255, 2, 0, 0, 0, 0, 0, 0, 
+    0, 0, 0, 0, 0, 0, 0, 2, 151, 20, 88, 131, 0, 0, 0, 0
+};
+
 TEST_F(PPPoETest, DefaultConstructor) {
     PPPoE pdu;
     EXPECT_EQ(1, pdu.version());
@@ -26,6 +50,57 @@ TEST_F(PPPoETest, DefaultConstructor) {
     EXPECT_EQ(0, pdu.payload_length());
 }
 
+TEST_F(PPPoETest, ConstructorFromSessionBuffer) {
+    PPPoE pdu(session_packet, sizeof(session_packet));
+    EXPECT_EQ(1, pdu.version());
+    EXPECT_EQ(1, pdu.type());
+    EXPECT_EQ(0x00, pdu.code());
+    EXPECT_EQ(0x62, pdu.session_id());
+    EXPECT_EQ(21, pdu.payload_length());
+    EXPECT_EQ(0U, pdu.tags().size());
+    
+    const RawPDU* raw = pdu.find_pdu<RawPDU>();
+    ASSERT_TRUE(raw != NULL);
+    EXPECT_EQ(21U, raw->payload_size());
+}
+
+TEST_F(PPPoETest, ConstructorFromFullSessionBuffer) {
+    EthernetII eth(full_session_packet, sizeof(full_session_packet));
+    const PPPoE& pdu = eth.rfind_pdu<PPPoE>();
+    EXPECT_EQ(1, pdu.version());
+    EXPECT_EQ(1, pdu.type());
+    EXPECT_EQ(0x00, pdu.code());
+    EXPECT_EQ(0x62, pdu.session_id());
+    EXPECT_EQ(21, pdu.payload_length());
+    EXPECT_EQ(0U, pdu.tags().size());
+    
+    const RawPDU* raw = pdu.find_pdu<RawPDU>();
+    ASSERT_TRUE(raw != NULL);
+    EXPECT_EQ(21U, raw->payload_size());
+
+    PDU::serialization_type buffer = eth.serialize();
+    EXPECT_EQ(
+        PDU::serialization_type(
+            full_session_packet, 
+            full_session_packet + sizeof(full_session_packet)
+        ),
+        buffer
+    );
+}
+
+TEST_F(PPPoETest, ConstructorFromFullSessionBuffer2) {
+    EthernetII eth(full_session_packet2, sizeof(full_session_packet2));
+
+    PDU::serialization_type buffer = eth.serialize();
+    EXPECT_EQ(
+        PDU::serialization_type(
+            full_session_packet2, 
+            full_session_packet2 + sizeof(full_session_packet2)
+        ),
+        buffer
+    );
+}
+
 TEST_F(PPPoETest, ConstructorFromBuffer) {
     PPPoE pdu(expected_packet, sizeof(expected_packet));
     EXPECT_EQ(1, pdu.version());
@@ -36,23 +111,23 @@ TEST_F(PPPoETest, ConstructorFromBuffer) {
     EXPECT_EQ(3U, pdu.tags().size());
     
     EXPECT_EQ("", pdu.service_name());
-    ASSERT_TRUE(pdu.search_tag(PPPoE::SERVICE_NAME));
+    ASSERT_TRUE(pdu.search_tag(PPPoE::SERVICE_NAME) != NULL);
 }
 
 TEST_F(PPPoETest, StackedOnEthernet) {
     EthernetII eth = EthernetII() / PPPoE();
     PDU::serialization_type buffer = eth.serialize();
-    EthernetII eth2(&buffer[0], buffer.size());
-    ASSERT_TRUE(eth2.find_pdu<PPPoE>());
+    EthernetII eth2(&buffer[0], (uint32_t)buffer.size());
+    ASSERT_TRUE(eth2.find_pdu<PPPoE>() != NULL);
 }
 
 TEST_F(PPPoETest, StackedOnEthernetSerializationWithTags) {
     PPPoE pdu(expected_packet, sizeof(expected_packet));
     EthernetII eth = EthernetII() / pdu;
     PDU::serialization_type buffer = eth.serialize();
-    EthernetII eth2(&buffer[0], buffer.size());
+    EthernetII eth2(&buffer[0], (uint32_t)buffer.size());
     PPPoE* unserialized = eth2.find_pdu<PPPoE>();
-    ASSERT_TRUE(unserialized);
+    ASSERT_TRUE(unserialized != NULL);
     EXPECT_EQ(
         PPPoE::serialization_type(expected_packet, expected_packet + sizeof(expected_packet)),
         unserialized->serialize()
