@@ -12,7 +12,7 @@ class SNAPTest : public testing::Test {
 public:
     static const uint8_t expected_packet[];
     
-    void test_equals(const SNAP &snap1, const SNAP &snap2);
+    void test_equals(const SNAP& snap1, const SNAP& snap2);
 };
 
 const uint8_t SNAPTest::expected_packet[] = {
@@ -87,8 +87,8 @@ TEST_F(SNAPTest, ClonePDU) {
     snap1.eth_type(0xfab1);
     snap1.org_code(0xfab1c3);
     snap1.control(0x1);
-    SNAP *snap2 = static_cast<SNAP*>(snap1.clone());
-    ASSERT_TRUE(snap2);
+    SNAP* snap2 = static_cast<SNAP*>(snap1.clone());
+    ASSERT_TRUE(snap2 != NULL);
     test_equals(snap1, *snap2);
     
     delete snap2;
@@ -104,11 +104,11 @@ TEST_F(SNAPTest, ConstructorFromBuffer) {
     EXPECT_EQ(0x0800, snap1.eth_type()); 
     EXPECT_EQ(1U, snap1.org_code()); 
     
-    SNAP snap2(&buffer[0], buffer.size());
+    SNAP snap2(&buffer[0], (uint32_t)buffer.size());
     test_equals(snap1, snap2);
 }
 
-void SNAPTest::test_equals(const SNAP &snap1, const SNAP &snap2) {
+void SNAPTest::test_equals(const SNAP& snap1, const SNAP& snap2) {
     EXPECT_EQ(snap1.dsap(), snap2.dsap());
     EXPECT_EQ(snap1.ssap(), snap2.ssap());
     EXPECT_EQ(snap1.control(), snap2.control());
